'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.installShellCommand = undefined;

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _path = require('path');

var _electron = require('electron');

var _spawnRx = require('spawn-rx');

var _fs = require('../utils/fs');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const fs = require('fs');

const getStartCommand = () => {
  const subdir = process.platform === 'darwin' ? 'MacOS' : '';
  const ext = process.platform === 'win32' ? '.exe' : '';
  const win = process.platform === 'win32' ? 'win' : '';
  const dir = (0, _path.join)(process.resourcesPath, '..', subdir);

  const nteractPath = (0, _path.join)(dir, `nteract${ ext }`);
  const electronPath = (0, _path.join)(dir, `electron${ ext }`);

  if (fs.existsSync(nteractPath)) {
    return [nteractPath, '', (0, _path.join)(process.resourcesPath, 'bin', win)];
  } else if (fs.existsSync(electronPath)) {
    // Developer install
    const rootDir = dir.split('node_modules')[0];
    return [electronPath, rootDir, (0, _path.join)(rootDir, 'bin', win)];
  }
  return [null, null, null];
};

const setWinPathObservable = (exe, rootDir, binDir) => {
  // Remove duplicates because SETX throws a error if path is to long
  const env = process.env.PATH.split(';').filter(item => !/nteract/.test(item)).filter((item, index, array) => array.indexOf(item) === index);
  env.push(binDir);
  const envPath = env.join(';');
  return _Rx2.default.Observable.merge((0, _spawnRx.spawn)('SETX', ['PATH', `${ envPath }`]), (0, _spawnRx.spawn)('SETX', ['NTERACT_EXE', exe]), (0, _spawnRx.spawn)('SETX', ['NTERACT_DIR', rootDir]));
};

const installShellCommandsObservable = (exe, rootDir, binDir) => {
  if (process.platform === 'win32') {
    return setWinPathObservable(exe, rootDir, binDir);
  }
  const envFile = (0, _path.join)(binDir, 'nteract-env');
  return (0, _fs.writeFileObservable)(envFile, `NTERACT_EXE="${ exe }"\nNTERACT_DIR="${ rootDir }"`).flatMap(() => {
    const target = (0, _path.join)(binDir, 'nteract.sh');
    return (0, _fs.createSymlinkObservable)(target, '/usr/local/bin/nteract').catch(() => {
      const dest = (0, _path.join)(process.env.HOME, '.local/bin/nteract');
      return (0, _fs.createSymlinkObservable)(target, dest);
    });
  });
};

const installShellCommand = exports.installShellCommand = () => {
  const [exe, rootDir, binDir] = getStartCommand();
  if (!exe) {
    _electron.dialog.showErrorBox('nteract application not found.', 'Could not locate nteract executable.');
    return;
  }

  installShellCommandsObservable(exe, rootDir, binDir).subscribe(() => {}, err => _electron.dialog.showErrorBox('Could not write shell script.', err.message), () => _electron.dialog.showMessageBox({
    title: 'Command installed.',
    message: 'The shell command "nteract" is installed.',
    detail: 'Get help with "nteract --help".',
    buttons: ['OK']
  }));
};