'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPath = getPath;
exports.deferURL = deferURL;
exports.launch = launch;
exports.launchNewNotebook = launchNewNotebook;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _electron = require('electron');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let launchIpynb;

function getPath(url) {
  const nUrl = url.substring(url.indexOf('static'), _path2.default.length);
  return _path2.default.join(__dirname, '..', '..', nUrl.replace('static/', ''));
}

function deferURL(event, url) {
  event.preventDefault();
  if (!url.startsWith('file:')) {
    _electron.shell.openExternal(url);
  } else if (url.endsWith('.ipynb')) {
    launchIpynb(getPath(url));
  }
}

const iconPath = _path2.default.join(__dirname, '..', '..', 'static', 'icon.png');

const initContextMenu = require('electron-context-menu');

// Setup right-click context menu for all BrowserWindows
initContextMenu();

function launch(filename) {
  let win = new _electron.BrowserWindow({
    width: 800,
    height: 1000,
    icon: iconPath,
    title: 'nteract'
  });

  const index = _path2.default.join(__dirname, '..', '..', 'static', 'index.html');
  win.loadURL(`file://${ index }`);

  win.webContents.on('did-finish-load', () => {
    if (filename) {
      win.webContents.send('main:load', filename);
    }
    win.webContents.send('main:load-config');
  });

  win.webContents.on('will-navigate', deferURL);

  // Emitted when the window is closed.
  win.on('closed', () => {
    win = null;
  });
  return win;
}
launchIpynb = launch;

function launchNewNotebook(kernelSpecName) {
  const win = launch();
  win.webContents.on('did-finish-load', () => {
    win.webContents.send('main:new', kernelSpecName);
  });
  return win;
}